;#<![CDATA[
;#
;######################################################################################
;#  This file defines the following flash functions
;#  
;#  massEraseTarget - Entirely erases the target.
;#                    The target should be left in an unsecured state.
;#
;#  isUnsecure - indicates if the target is secured in some fashion (read/write protected)
;#               Returns TCL_OK if NOT secured
;#
;#  initFlash - initialises the target for flash programing (if needed)
;#
;#  initTarget - initialises the target for general use
;#
;#  In addition the script may do any once-only initialisation such as setting global symbols
;#  when initially loaded into the TCL environment.
;#

;######################################################################################
;#
;#
proc loadSymbols {} {
   # LittleEndian format for writing numbers to memory
   setbytesex  littleEndian
   defaultMemorySpace X

   set ::DSC_FSEC_SEC_MASK     0x03   ;# Security bits
   set ::DSC_FSEC_SEC_UNSEC    0x02   ;# Security bits for unsecured device
   set ::DSC_FSEC_SEC_KEYEN    0x80   ;# Backdoor Key enable
   set ::DSC_FSEC_SEC_FNORED   0x40   ;# Vector redirection disable
   set ::DSC_FSEC_UNSEC_VALUE  0xFFFE ;# Value to use when unsecuring (0xFF:NVSEC value)
      
   set ::HFM_FCLKD             x:0xF400
   set ::HFM_CR                x:0xF401
   set ::HFM_SECH              x:0xF402
   set ::HFM_SECL              x:0xF403
   set ::HFM_PROT              x:0xF410
   set ::HFM_USTAT             x:0xF413
   set ::HFM_CMD               x:0xF414
   set ::HFM_DATA              x:0xF418
   set ::HFM_IFR_OPT0          x:0xF41A
   set ::HFM_TST_SIG           x:0xF41D

   set ::HFM_USTAT_CBEIF       0x80
   set ::HFM_USTAT_CCIF        0x40
   set ::HFM_USTAT_PVIOL       0x20
   set ::HFM_USTAT_ACCERR      0x10
   set ::HFM_USTAT_BLANK       0x04

   set ::HFM_CMD_ERASE_VER     0x05
   set ::HFM_CMD_CALC_SIG      0x06
   set ::HFM_CMD_WORD_PROG     0x20
   set ::HFM_CMD_SECTOR_ERASE  0x40
   set ::HFM_CMD_MASS_ERASE    0x41
   set ::HFM_CMD_CALC_IFR_SIG  0x66

   set ::DSC_PRDIV8            0x40
   
   set ::FLASH_ADDRESS         p:0x2000  ;# Dummy location in Flash
   
   set ::BUS_FREQUENCY         4000
}

;######################################################################################
;#
;#
proc initTarget { flashAddress } {
   ;#  puts "initTarget {}"
   
   set ::FLASH_ADDRESS   p:$flashAddress
}

;######################################################################################
;#
;#  busFrequency - Target bus busFrequency in kHz
;#
proc initFlash { busFrequency } {
   ;#  puts "initFlash {}"
   
   set ::BUS_FREQUENCY $busFrequency

   ;# Set up Flash
   set cfmclkd [calculateFlashDivider $busFrequency]
   ww $::HFM_FCLKD $cfmclkd   ;# Flash divider
   ww $::HFM_CR    0x0000     ;# unprotect Flash
   ww $::HFM_PROT  0x0000     ;# unprotect Flash
}

;######################################################################################
;#
;#  busFrequency - Target bus busFrequency in kHz
;#
proc calculateFlashDivider { busFrequency } {
;# NOTES:
;# According to data sheets the Flash uses the Oscillator clock for timing but 
;# it is also influenced by the bus period.
;#
;# This code assumes that oscillator clock = 2 * bus clock
;#
   ;#  puts "calculateFlashDivider {}"
   if { [expr $busFrequency < 1000] } {
      error "Clock too low for flash programming"
   }
   set cfmclkd 0
   if { [expr $busFrequency > 6400] } {
      set cfmclkd [expr $::DSC_PRDIV8 + round(floor(0.249999+1.25*($busFrequency/1000.0)))]
      set flashClk [expr $busFrequency / (4*(($cfmclkd&0x3F)+1))]
   } else {
      set cfmclkd [expr round(floor(0.99999+($busFrequency/100.0)))+1]
      set flashClk [expr (2*$busFrequency) / (($cfmclkd&0x3F)+1)]
   }
   ;#  puts "cfmclkd = $cfmclkd, flashClk = $flashClk"
   if { [expr ($flashClk<150)] } {
      error "Not possible to find suitable flash clock divider"
   }      
   return $cfmclkd
}

;######################################################################################
;#
;#  cmd     - command to execute
;#  address - flash address to use
;#  value   - data value to use
;#
proc executeFlashCommand { cmd address value } {

   ;#  puts "executeFlashCommand {}"
   
   ww $::HFM_USTAT    $::HFM_USTAT_PVIOL      ;# Clear PVIOL/ACCERR
   ww $::HFM_USTAT    $::HFM_USTAT_ACCERR     ;# Clear PVIOL/ACCERR
   ww $address        $value                  ;# Write to Program flash to buffer address and data
   ww $::HFM_CMD      $cmd                    ;# Write command
   ww $::HFM_USTAT    $::HFM_USTAT_CBEIF      ;# Clear CBEIF to execute the command 
   
   ;# Wait for command completion
   set flashBusy 1
   set retry 0
   while { $flashBusy } {
      after 20
      set status [rw $::HFM_USTAT]
      set flashBusy  [expr ($status & $::HFM_USTAT_CCIF) == 0x00]
      set flashError [expr ($status & ($::HFM_USTAT_PVIOL|$::HFM_USTAT_ACCERR))]
      if [expr $flashError != 0] {
         break;
      }
      if [expr $retry == 20] {
         break;
      }
      incr retry
   }
   if [ expr ($flashError || ($retry>=20)) ] {
      ;#  puts [ format "Flash command error HFM_USTAT=0x%02X, retry=%d" $status $retry ]
      error "Flash command failed"
   }
}

;######################################################################################
;#  Target is erased & unsecured
;#  This code isn't much use since it can only be used when the device is unsecured
proc massEraseTarget { } {

   ;#  puts "massEraseTarget{}"
   
   ;# No initial connect as may fail.  Assumed done by caller.

   ;# Mass erase flash (& EEPROM)
   initFlash $::BUS_FREQUENCY
   executeFlashCommand  $::HFM_CMD_MASS_ERASE $::FLASH_ADDRESS  0xFFFF

   return
   ;# Device is now Blank but may not be unsecured
   ;#  puts "Checking if target is already unsecured"
   set securityValue [rb $::DSC_FSEC]
   if [ expr ( $securityValue & $::DSC_FSEC_SEC_MASK ) == $::DSC_FSEC_SEC_UNSEC ] {
      ;# Target was originally unsecured - just return now
      ;#  puts "Target was originally unsecured"
      return
   }
   ;# Reset & re-connect for changes to be effected.  Device is partially unsecured
   ;# BDMSTS.UNSEC == 1
   reset s h
   connect   ;# shouldn't fail
   
   ;# Program security location to unsecured value
   initTarget $::FLASH_ADDRESS $::EEPROM_ADDRESS
   initFlash $::BUS_FREQUENCY
   executeFlashCommand  $::HFM_CMD_WORD_PROG $::DSC_NVSEC $::DSC_FSEC_UNSEC_VALUE
   
   ;# Reset to have unsecured (finally) but not blank device
   reset s h
   connect   ;# shouldn't fail

   ;# Confirm unsecured
   ;#  puts "Checking if target is unsecured"
   set securityValue [rb $::DSC_FSEC]
   if [ expr ( $securityValue & $::DSC_FSEC_SEC_MASK ) != $::DSC_FSEC_SEC_UNSEC ] {
      ;#  puts "Target failed to unsecure"
      error "Target failed to unsecure"
      return
   }
   ;# Erase security location so device is unsecured and blank!
   initTarget $::FLASH_ADDRESS $::EEPROM_ADDRESS
   initFlash $::BUS_FREQUENCY
   executeFlashCommand  $::HFM_CMD_SECTOR_ERASE $::DSC_NVSEC 0xFFFF

   ;# Flash is now Blank and unsecured
}

;######################################################################################
;#
proc isUnsecure { } {
   return
   ;#  puts "Checking if unsecured"
   set securityValue [rb $::DSC_FSEC]

   if [ expr ( $securityValue & $::DSC_FSEC_SEC_MASK ) != $::DSC_FSEC_SEC_UNSEC ] {
      error "Target is secured"
   }
}

;######################################################################################
;# Actions on initial load
;#
loadSymbols

;#]]>